[inherit('common', 'phttp',  'pjson')]
module pSolr(input, output);

function replace(s, fnd, rpl : pstr) : pstr;

var
   res : pstr;
   i : integer;

begin
   res := '';
   i := 1;
   while i <= length(s) do begin
      if substr(s, i, length(fnd)) = fnd then begin
         res := res + rpl;
         i := i + length(fnd);
      end else begin
         res := res + s[i];
         i := i + 1;
      end;
   end;
   replace := res;
end;

type
   solr_ctx = record
                 host : pstr;
                 port : integer;
                 collection : pstr;
              end;
   solr_ctxptr = ^solr_ctx;
   solr_query_res = cJSON_ptr;

function psolr_setup(host : pstr; port : integer; collection : pstr) : solr_ctxptr;

var
   res : solr_ctxptr;

begin
   new(res);
   res^.host := host;
   res^.port := port;
   res^.collection := collection;
   psolr_setup := res;
end;

function psolr_query_options(ctx : solr_ctxptr; query : pstr; options : pstr) : solr_query_res;

var
   con : http;
   doc : pstr;
   res : cJSON_ptr;

begin
   con := http_get(fix(ctx^.host),
                   ctx^.port,
                   '/solr/' + ctx^.collection + '/select?wt=json&q=' + replace(query, ' ', '+') + options,
                   'application/json');
   if (http_numcode(con) div 100) <> 2 then begin
      writeln('HTTP error : ', http_numcode(con):1, ' ', http_txtcode2(con));
   end;
   http_recv_all(con, doc.body, doc.length);
   http_close(con);
   res := pJSON_parse(doc);
   res := pJSON_GetObjectItem(res, 'response');
   res := pJSON_GetObjectItem(res, 'docs');
   psolr_query_options := res;
end;

function psolr_query_df_rows(ctx : solr_ctxptr; query : pstr; df : pstr; rows : integer) : solr_query_res;

begin
   psolr_query_df_rows := psolr_query_options(ctx, query, '&df=' + df + '&rows=' + int2str(rows));
end;

function psolr_query_rows(ctx : solr_ctxptr; query : pstr; rows : integer) : solr_query_res;

begin
   psolr_query_rows := psolr_query_options(ctx, query, '&rows=' + int2str(rows));
end;

function psolr_query_df(ctx : solr_ctxptr; query : pstr; df : pstr) : solr_query_res;

begin
   psolr_query_df := psolr_query_options(ctx, query, '&df=' + df);
end;

function psolr_query(ctx : solr_ctxptr; query : pstr) : solr_query_res;

begin
   psolr_query := psolr_query_options(ctx, query, '');
end;

function psolr_hits(query_res : solr_query_res) : integer;

begin
   psolr_hits := pJSON_GetArraySize(query_res);
end;

function psolr_get_field(query_res : solr_query_res; docix : integer; field : pstr) : pstr;

begin
   psolr_get_field := pJSON_StringValue(pJSON_GetObjectItem(pJSON_GetArrayItem(query_res, docix), field));
end;

procedure psolr_delete(ctx : solr_ctxptr; query : pstr);

var
   con : http;
   json, temp : cJSON_ptr;
   txt : pstr;

begin
   temp := pJSON_CreateObject;
   pJSON_AddItemToObject(temp, 'query', pJSON_CreateString(query));
   json := pJSON_CreateObject;
   pJSON_AddItemReferenceToObject(json, 'delete', temp);
   txt := pJSON_PrintUnformatted(json);
   con := http_post(fix(ctx^.host),
                        ctx^.port,
                        '/solr/' + ctx^.collection + '/update?commit=true',
                        'application/json',
                        'application/json',
                        fix(txt));
   if (http_numcode(con) div 100) <> 2 then begin
      writeln('HTTP error : ', http_numcode(con):1, ' ', http_txtcode2(con));
   end;
   http_close(con);
end;

procedure psolr_insert(ctx : solr_ctxptr; id : integer; kv : [list] pstr);

var
   con : http;
   json : cJSON_ptr;
   txt : pstr;
   i : integer;

begin
   json := pJSON_CreateObject;
   pJSON_AddItemToObject(json, 'id',  pJSON_CreateNumber(id));
   for i := 1 to (argument_list_length(kv) div 2) do begin
      pJSON_AddItemToObject(json, argument(kv, 2*i-1),  pJSON_CreateString(argument(kv, 2*i)));
   end;
   txt := pJSON_PrintUnformatted(json);
   con := http_post(fix(ctx^.host),
                    ctx^.port,
                    '/solr/' + ctx^.collection + '/update/json/docs?commit=true',
                    'application/json',
                    'application/json',
                    fix(txt));
   if (http_numcode(con) div 100) <> 2 then begin
      writeln('HTTP error : ', http_numcode(con):1, ' ', http_txtcode2(con));
   end;
   http_close(con);
end;

end.
